/*
 *  Copyright (c) 2009 Luca Abeni
 *
 *  This is free software; see gpl-3.0.txt
 */

#include <stdio.h>
#include <stdint.h>

enum states {NORMAL, RESYNCH_0, RESYNCH_1};

int data_read(FILE *f, int *size, int64_t *pts)
{
  int res, cnt, d;

  res = fscanf(f, "%d %d %Ld %d\n", &cnt, size, pts, &d);
  //if (res < 0) {
  //  return -2;
  //}
  if (res != 4) {
    return 0;
  }

  return res;
}

int main(int argc, char *argv[])
{
  FILE *reference, *corrupted;
  int done, ref_cnt, corr_cnt;
  enum states state;
  int64_t ref_pts=0, corr_pts=0;
  int corr_size = -1;

  reference = fopen(argv[1], "r");
  corrupted = fopen(argv[2], "r");
  if (reference == NULL || corrupted == NULL) {
    fprintf(stderr, "Error opening an input file!\n");

    return -1;
  }

  done = 0;
  ref_cnt = 0; corr_cnt = 0; 
  state = NORMAL;
  while (!done) {
    int res, ref_size, corr_size_next;
    int drop_start;

    res = data_read(reference, &ref_size, &ref_pts);
    if (res < 0) {
      fprintf(stderr, "[%d]Error on reference: %d\n", ref_cnt, res);

      return -1;
    } else if (res == 0) {
      done = 1;
    }
    ref_cnt++;

    if ((state == NORMAL) || (state == RESYNCH_1)) {
      if (corr_size < 0) {
        res = data_read(corrupted, &corr_size, &corr_pts);
      }
      if (ref_size != corr_size) {
        if (state == NORMAL) {
          fprintf(stderr, "Missing frame from %d to ", ref_cnt);
          data_read(corrupted, &corr_size_next, &corr_pts);
          drop_start = ref_cnt;
          state = RESYNCH_0;
        }
      } else {
        corr_size = -1;
        corr_cnt++;
      }
    }
    if (state == RESYNCH_0) {
fprintf(stderr, "%d or %d = %d?\n", corr_size, corr_size_next, ref_size);
      if (ref_size == corr_size) {
        fprintf(stderr, "%d\n", ref_cnt);
        state = NORMAL;
        printf("Duplicate %d %d times\n", corr_cnt - 1, ref_cnt - drop_start);
        corr_size = corr_size_next;
//        data_read(reference, &ref_size, &ref_pts);
        corr_cnt++;
//        ref_cnt++;
      } else if (ref_size == corr_size_next) {
        fprintf(stderr, "%d\n", ref_cnt);
        state = NORMAL;
        printf("Duplicate %d %d times\n", corr_cnt, ref_cnt - drop_start - 1);
        corr_size = -1;
        corr_cnt += 2;
      }
    }
  }

  return 0;
}
