#!/usr/bin/perl
 ###########################################
 #  Copyright (c) 2009 Csaba Kiraly        #
 #                                         #
 #  This is free software; see gpl-3.0.txt #
 ###########################################

use strict;
use Statistics::Descriptive;
use POSIX qw(ceil floor);
use Switch;

my $gl_inputfolder;

($ENV{'PSNR_TOOLS'}) or (print "\$PSNR_TOOLS not defined\n" and die);
my $PSNR_TOOLS=$ENV{'PSNR_TOOLS'};
($ENV{'PSNR_TMPPREFIX'}) or (print "\$PSNR_TMPPREFIX not defined\n" and die);
my $PSNR_TMPPREFIX=$ENV{'PSNR_TMPPREFIX'};

#a value of 'src' for the peer creates an empty lost file
sub peer_chunks_lost ($) {
  my ($peer)=@_;

  my @losts;
  if ($peer=='src') {
    @losts=`echo -n > lost.txt`;
  } else {
    @losts=`$PSNR_TOOLS/chunks_lost.sh $peer $gl_inputfolder| tee lost.txt`;
  }
  chomp @losts;
  return @losts;
}

sub peer_qoe ($$) {
  my ($peer,$chunksize)=@_;

  my @losts = peer_chunks_lost($peer);

  my %qoe;
  #$qoe{ 'chunkloss' } = peer_chunkloss($peer,@losts);

  my $q;
  $q=`$PSNR_TOOLS/psnr.sh calc $chunksize lost.txt`;

  $q =~ m/psnr: ([\d\.]+)/;
  $qoe{ 'psnr' } = $1;

  $q =~ m/ssim: ([\d\.]+)/;
  $qoe{ 'ssim' } = $1;

  $q =~ m/bytes: ([\d\.]+)/;
  $qoe{ 'bytes' } = $1;

  $q =~ m/chunks: ([\d\.]+)/;
  $qoe{ 'chunks' } = $1;

  $q =~ m/chunkslost: ([\d\.]+)/;
  $qoe{ 'chunkslost' } = $1;

  $qoe{ 'chunkloss' } = $qoe{ 'chunkslost' } / $qoe{ 'chunks' } ;

  return \%qoe;
}

#sub num_chunks() {
#  return 1000;
#}
#
#sub peer_chunkloss ($@) {
#  my ($peer,@losts)=@_;
#  return (scalar @losts) / num_chunks();
#}

sub peers_of_class ($) {
  my ($peerclass)=@_;

  my @peers=`$PSNR_TOOLS/peers_of_class.sh $peerclass $gl_inputfolder`;
  chomp @peers;
  return @peers;
}

sub print_stat($\%$) {
  my ($name,$qoes,$metric)=@_;

  my $stat = Statistics::Descriptive::Full->new();
  foreach my $peer (keys(%$qoes)) {
    $stat->add_data($qoes->{$peer}{$metric});
  }

  print "$name: ";
  print " mean=".$stat->mean(); 
  print " var=".$stat->variance();
  print " count=".$stat->count();
  print "\n";

  return $stat->mean();
}

sub print_stats(\%){
  my ($qoes)=@_;
  my %avgqoe;
  $avgqoe{'psnr'} = print_stat("  psnr(uncompressed->received)",%$qoes ,'psnr');
  $avgqoe{'ssim'} = print_stat("  ssim",%$qoes,'ssim');
  $avgqoe{'chunkloss'} = print_stat("  chunkloss",%$qoes,'chunkloss');
  print "\n";

  return \%avgqoe;
}

sub qoe($$){
  my ($chunksize,$every)=@_;

  my %allqoes;
  my @peerclasses=(1,2,3,4);

  foreach my $peerclass (@peerclasses) {

    print " Peer class = $peerclass\n";

    my @peers=peers_of_class($peerclass);

    my %qoes;
    my $everycounter=0;
    foreach my $peer (@peers) {
      if ($everycounter % $every == 0) {
        $qoes{$peer} = peer_qoe($peer,$chunksize);
      }
      $everycounter++;
    }
    print_stats(%qoes);
    @allqoes{keys %qoes} = values %qoes;
  }

  print " All peers\n";
  my $avgqoe = print_stats(%allqoes);

  return $avgqoe;
}

sub init ($$) {
  my ($video,$coder)=@_;
  my $tmp=`$PSNR_TOOLS/psnr.sh init $video "$coder"`;
}

sub calc ($$) {
  my ($inputfolder,$every)=@_;
  $gl_inputfolder=$inputfolder;

  my $srcqoe = peer_qoe('src',0);
  print "psnr(uncompressed->compressed): ".$srcqoe->{'psnr'}."\n";
  print "ssim(uncompressed->compressed): ".$srcqoe->{'ssim'}."\n";

  my $chunksize = ceil($srcqoe->{'bytes'}/$srcqoe->{'chunks'}); 

  print "\nQuality with GOP size chunks (avg chunk size = $srcqoe->{'bytes'}/$srcqoe->{'chunks'} bytes )\n";
  my $avgqoe_gop = qoe(0,$every);
  print "\nQuality with interleaved GOP size chunks (avg chunk size = $srcqoe->{'bytes'}/$srcqoe->{'chunks'} bytes )\n";
  my $avgqoe_interleavedgop = qoe(-1,$every);
  print "\nQuality with $chunksize bytes chunks\n";
  my $avgqoe_nogop = qoe($chunksize,$every);
  print "PSNRresults $srcqoe->{'psnr'} $avgqoe_gop->{'psnr'} $avgqoe_interleavedgop->{'psnr'} $avgqoe_nogop->{'psnr'}".
                   " $srcqoe->{'ssim'} $avgqoe_gop->{'ssim'} $avgqoe_interleavedgop->{'ssim'} $avgqoe_nogop->{'ssim'}".
  "\n";
}

#--------------------------------------------------

if ($ARGV[0] eq 'init') {
  ($#ARGV + 1 == 3) or die;
  my $video=$ARGV[1];
  my $coder=$ARGV[2];
  init($video,$coder);
} elsif ($ARGV[0] eq 'calc') {
  ($#ARGV + 1 == 3) or die;
  my $inputfolder=$ARGV[1];
  my $every=$ARGV[2]; #calculate values only for every "everty"th of peers
  calc($inputfolder,$every);
} else {
  print "Usage: ... see source\n";
}

