/*
 *  Copyright (c) 2009 Luca Abeni
 *
 *  This is free software; see gpl-3.0.txt
 */

#include <libavformat/avformat.h>

#include "input.h"

void *packet_new(int i)
{
  static AVPacket pkt;

  pkt.stream_index = i;
  pkt.data = NULL;
  pkt.size = 0;
  pkt.pts = AV_NOPTS_VALUE;

  return &pkt;
}

void close_input_stream(void *s)
{
    av_close_input_file(s);
}

void *open_input_stream(const char *fname)
{
    AVFormatContext *s;
    int res, i;

    res = av_open_input_file(&s, fname, NULL, 0, NULL);
    if (res < 0) {
        fprintf(stderr, "Error opening %s: %d\n", fname, res);

        return NULL;
    }

    res = av_find_stream_info(s);
    if (res < 0) {
        fprintf(stderr, "Cannot find codec parameters for %s\n", fname);

        return NULL;
    }

    for (i = 0; i < s->nb_streams; i++) {
        AVCodec *c;
	
        c = avcodec_find_decoder(s->streams[i]->codec->codec_id);
        if (c == NULL) {
            fprintf(stderr, "Unsupported codec %x for input stream %d\n",
                        s->streams[i]->codec->codec_id, i);
            return NULL;
	}
        res = avcodec_open(s->streams[i]->codec, c);
	if (res < 0) {
            fprintf(stderr, "Error while opening codec for input stream #%d\n", i);
            return NULL;
	}
fprintf(stderr, "TB: %d/%d\n", s->streams[i]->time_base.num, s->streams[i]->time_base.den);
    }

    dump_format(s, 0, fname, 0);

    return s;
}

void *read_input_stream(void *s)
{
  static AVPacket pkt;
  int res;
  
  res = av_read_frame(s, &pkt);
  if (res < 0) {
    return NULL;
  }
//fprintf(stderr, "R: %Lu\t", pkt.pts);

  return &pkt;
}
