/*
 *  Copyright (c) 2009 Luca Abeni
 *
 *  This is free software; see gpl-3.0.txt
 */

#include <stdint.h>
#include <stdlib.h>
#include <stdio.h>

#include "stream.h"
#include "input.h"
#include "codec.h"
#include "output.h"

struct stream {
  void *in_s;
  void *out_s;
  struct dups *duplicates;
};

void *stream_open(const char *fname, const char *outfname, struct dups *d)
{
  struct stream *c;
  int i, res;

  codec_init();

  c = malloc(sizeof(struct stream));
  if (c == NULL) {
    perror("Malloc");

    return NULL;
  }

  c->in_s = open_input_stream(fname);
  if (c->in_s == NULL) {
    fprintf(stderr, "Cannot open input stream\n");

    goto err1;
  }
  c->out_s = alloc_output(outfname);
  for (i = 0; i < num_streams(c->in_s); i++) {
    res = prepare_output(c->out_s, c->in_s, i);
  }
  if (finalize_output(c->out_s) < 0) {
    goto err2;
  }
  c->duplicates = d;

  return c;

err2:
  close_output(c->out_s);
  close_input_stream(c->in_s);

err1:
  free(c);

  return NULL;
}

void *stream_read(void *h)
{
  struct stream *s = h;

  return read_input_stream(s->in_s);
}

int stream_process(void *h, void *pkt)
{
  struct stream *s = h;
  static int cnt, dup_cnt;
  static void *out_pkt;
  int res = 0;

  if (stream_copy(s->in_s, pkt_stream(pkt))) {
    ts_copy(pkt, s->out_s, pkt, s->in_s, pkt_stream(pkt));
    res = write_output(s->out_s, pkt); 
  } else {
    void *frame_handle;

    frame_handle = decode(s->in_s, pkt);
    if (frame_handle) {
      frame_pts_null(frame_handle);
      out_pkt = encode(s->out_s, frame_handle, pkt_stream(pkt));
    }

    if (out_pkt) {
      res = write_output(s->out_s, out_pkt);
      if (s->duplicates[dup_cnt].frame == cnt) {
fprintf(stderr, "Dup?\n");
        int i;

        for (i = 0; i < s->duplicates[dup_cnt].amount; i++) {
fprintf(stderr, "Writing duplicate %d %d\n", cnt, i);
          res += write_output(s->out_s, out_pkt);
        }
        dup_cnt++;
      }
    
//fprintf(stderr, "Read: %d\n", cnt++);
      cnt++;
    }
  }

  return res;
}

void stream_close(void *h)
{
  struct stream *s = h;
  int done = 0;

  while (!done) {
    int res;
    void *p = packet_new(0);

    res = stream_process(h, p);
    if (res == 0) {
      done = 1;
    }
  }

  close_output(s->out_s);
}
